/*
 * Created on Aug 22, 2005
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * History
 * v1.0 - Aug 22, 2005 - Initial Release
 */

/**
 * @author kuwanger
 *
 * A very simple (read, incomplete) Little Endian compatible "OutputStream"
 */

import java.io.DataInputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.FileNotFoundException;

public class LESimpleDataInputStream {
	private DataInputStream in;
	boolean eof = false;
	
	LESimpleDataInputStream(String filename) throws FileNotFoundException
	{
		in = new DataInputStream(new FileInputStream(filename));
	}
	
	void close() throws IOException
	{
		in.close();
	}
	
	boolean eof() throws IOException
	{
		return (in.available() == 0);
	}
	
	byte readByte() throws IOException
	{
		byte[] bs = new byte[1];
		
		in.read(bs, 0, 1);
		return bs[0];
	}

	char readChar() throws IOException
	{
		byte[] bs = new byte[2];
		int[] positive = new int[2];
		
		in.read(bs, 0, 2);
		positive[0] = (bs[0] < 0) ? bs[0] + 256 : bs[0];
		positive[1] = (bs[1] < 0) ? bs[1] + 256 : bs[1];
		return (char) (positive[0] | (positive[1] << 8));
	}
	
	int readInt() throws IOException
	{
		byte[] bs = new byte[4];
		int[] positive = new int[4];
		
		in.read(bs, 0, 4);
		positive[0] = (bs[0] < 0) ? bs[0] + 256 : bs[0];
		positive[1] = (bs[1] < 0) ? bs[1] + 256 : bs[1];
		positive[2] = (bs[2] < 0) ? bs[2] + 256 : bs[2];
		positive[3] = (bs[3] < 0) ? bs[3] + 256 : bs[3];
		return (positive[0] | (positive[1]<<8) |
			(positive[2]<<16) + (positive[3]<<24));
	}
}
