/*
 * Created on Aug 22, 2005
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * History
 * v1.0 - Aug 22, 2005 - Initial Release
 */

/**
 * @author kuwanger
 *
 * LZSS (GBA LZ77) variant.  This is VRAM safe.  Output is 8-bit chars.  Yes,
 * byte[] would probably make more sense.
 */

public class LZ77 {
	static final int MAXWINDOW = 4096;
	
	private static SubChars search(byte[] buffer, int window_index, int data_index)
	{
		SubChars result = new SubChars();
		int count;
		
		for (int i = window_index; i < data_index - 1; i++)
		{
			count = 0;
			while (count < 18 &&
					data_index+count < buffer.length &&
					buffer[i+count] == buffer[data_index+count])
				count++;
			if (count > result.length)
			{
				result.length = count;
				result.index = i;
			}
			if (count == 18)
				break;
		}
		
		return result;
	}
	
	static char[] Compress(char[] z)
	{
		byte[] b = new byte[z.length*2];
		
		for (int i = 0; i < z.length; i++)
		{
			b[i*2] = (byte) (z[i]%256);
			b[i*2+1] = (byte) (z[i]>>8);
		}
		
		return Compress(b);
	}
	
	static char[] Compress(byte[] z)
	{
		char[] compressed = new char[z.length*9/8+1+4];
		char cvalue;
		int flags = 4, data = 5, index = 0, window = 0, ormask = 127;
		int displacement, newlength;
		SubChars result;
		
		compressed[0] = 0x10;
		compressed[1] = (char) (z.length%256);
		compressed[2] = (char) ((z.length>>8)%256);
		compressed[3] = (char) ((z.length>>16)%256);
		
		compressed[flags] = 0;
		if (index < z.length)
		{
		compressed[data++] = (char) z[index++];
		
		ormask = 128;
		
		while(index < z.length)
		{
			ormask >>= 1;
			if (ormask == 0) {
				ormask = 128;
				flags = data++;
				compressed[flags] = 0;
			}
			
			window = (index < MAXWINDOW) ? 0 : index - MAXWINDOW;
			
			result = search(z, window, index);
			if (result.length > 2) {
				compressed[flags] |= ormask;
				displacement = index - result.index - 1;
				cvalue = (char) ((displacement >> 8) | (displacement << 8) | ((result.length - 3) << 4));
				compressed[data++] = (char) (cvalue%256);
				compressed[data++] = (char) (cvalue>>8);
				index += result.length;
			} else {
				compressed[data++] = (char) z[index++];
			}
		}
		}
		
		newlength = (data + 3) & ~3;
		
		char[] result_minimal = new char[newlength];
		for (int i = 0; i < data; i++)
			result_minimal[i] = compressed[i];
		
		return result_minimal;
	}
}
