/*
 * Created on Aug 22, 2005
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * History
 * v1.0 - Aug 22, 2005 - Initial Release
 */

/**
 * @author kuwanger
 *
 * 15 color palette (0 is transparent).  Use updatePalette to scan an image
 * block for new colors.
 */

import java.awt.image.BufferedImage;

class Palette {
	private final int MAXCOLORS = 16;	// 0 is transparent
	private int palette[] = new int[MAXCOLORS];
	private int index = 1;
	
	Palette ()
	{
		palette[0] = 0;
	}
	
	char[] getPaletteChars()
	{
		char[] raw = new char[MAXCOLORS];
		
		for (int i = 0; i < MAXCOLORS; i++)
		{
			raw[i] = (char) palette[i];
		}
		
		return raw;
	}
	
	private int rgb2bgr(int rgb)
	{
		int bgr;
		
		bgr = ((rgb >> 19) & 31) |
		(((rgb >> 11) & 31) << 5) |
		(((rgb >> 3) & 31) << 10);
		return (bgr == 0) ? (1 | (1 << 5) | (1 << 10)) : bgr;
	}
	
	void updatePalette(BufferedImage bi) throws TooManyColorsException
	{
		int c;
		
		for (int y = 0; y < 8; y++)
		{
			for (int x = 0; x < 8; x++)
			{
				update(bi.getRGB(x,y));
			}
		}
	}
	
	void update(int rgb) throws TooManyColorsException
	{
		int bgr = rgb2bgr(rgb);
		
		for (int temp = 1; temp < index; temp++)
		{
			if (bgr == palette[temp])
				return ;
		}
		if (index >= MAXCOLORS)
			throw new TooManyColorsException();
		
		palette[index++] = bgr;
	}
	
	int getByRGB(int rgb) throws InvalidRGBException
	{
		int temp = rgb2bgr(rgb);
		
		for (int i = 1; i < index; i++)
		{
			if (palette[i] == temp)
				return i;
		}
		
		for (int i = 1; i < index; i++)
			System.out.print(palette[i] + " ");
		System.out.println(" ");
		throw new InvalidRGBException();
	}
}
